@@ -103,7 +103,8 @@ if sys.version_info >= (3, 5):
 for key, impl in list(BIN_OP_IMPL.items()):
     BIN_OP_IMPL[key+'='] = impl
 
-def const_infer_binary_op(self, operator, other, context):
+def const_infer_binary_op(self, binop, other, context):
+    operator = binop.op
     for other in other.infer(context):
         if isinstance(other, nodes.Const):
             try:
@@ -122,7 +123,7 @@ def const_infer_binary_op(self, operator, other, context):
             yield other
         else:
             try:
-                for val in other.infer_binary_op(operator, self, context):
+                for val in other.infer_binary_op(binop, self, context):
                     yield val
             except AttributeError:
                 yield util.YES
@@ -130,8 +131,9 @@ nodes.Const.infer_binary_op = bases.yes_if_nothing_inferred(const_infer_binary_o
 
 
 
-def _multiply_seq_by_int(self, other, context):
+def _multiply_seq_by_int(self, binop, other, context):
     node = self.__class__()
+    node.parent = binop
     elts = []
     for elt in self.elts:
         infered = util.safe_infer(elt, context)
@@ -151,10 +153,12 @@ def _filter_uninferable_nodes(elts, context):
                 yield inferred
 
 
-def tl_infer_binary_op(self, operator, other, context):
+def tl_infer_binary_op(self, binop, other, context):
+    operator = binop.op
     for other in other.infer(context):
         if isinstance(other, self.__class__) and operator == '+':
             node = self.__class__()
+            node.parent = binop
             elts = list(_filter_uninferable_nodes(self.elts, context))
             elts += list(_filter_uninferable_nodes(other.elts, context))
             node.elts = elts
@@ -163,7 +167,7 @@ def tl_infer_binary_op(self, operator, other, context):
             if not isinstance(other.value, int):
                 yield util.YES
                 continue
-            yield _multiply_seq_by_int(self, other, context)
+            yield _multiply_seq_by_int(self, binop, other, context)
         elif isinstance(other, bases.Instance) and not isinstance(other, nodes.Const):
             yield util.YES
     # XXX else log TypeError
@@ -171,14 +175,15 @@ nodes.Tuple.infer_binary_op = bases.yes_if_nothing_inferred(tl_infer_binary_op)
 nodes.List.infer_binary_op = bases.yes_if_nothing_inferred(tl_infer_binary_op)
 
 
-def dict_infer_binary_op(self, operator, other, context):
+def dict_infer_binary_op(self, binop, other, context):
     for other in other.infer(context):
         if isinstance(other, bases.Instance) and isinstance(other._proxied, nodes.ClassDef):
             yield util.YES
         # XXX else log TypeError
 nodes.Dict.infer_binary_op = bases.yes_if_nothing_inferred(dict_infer_binary_op)
 
-def instance_infer_binary_op(self, operator, other, context):
+def instance_infer_binary_op(self, binop, other, context):
+    operator = binop.op
     try:
         methods = self.getattr(BIN_OP_METHOD[operator])
     except (exceptions.NotFoundError, KeyError):
@@ -250,7 +255,7 @@ def _resolve_looppart(parts, asspath, context):
 
 
 @bases.raise_if_nothing_inferred
-def for_assigned_stmts(self, node, context=None, asspath=None):
+def for_assigned_stmts(self, node=None, context=None, asspath=None):
     if asspath is None:
         for lst in self.iter.infer(context):
             if isinstance(lst, (nodes.Tuple, nodes.List)):
@@ -265,18 +270,25 @@ nodes.For.assigned_stmts = for_assigned_stmts
 nodes.Comprehension.assigned_stmts = for_assigned_stmts
 
 
-def mulass_assigned_stmts(self, node, context=None, asspath=None):
+def sequence_assigned_stmts(self, node=None, context=None, asspath=None):
     if asspath is None:
         asspath = []
-    asspath.insert(0, self.elts.index(node))
-    return self.parent.assigned_stmts(self, context, asspath)
+    try:
+        index = self.elts.index(node)
+    except ValueError:
+         util.reraise(exceptions.InferenceError(
+             'Tried to retrieve a node {node!r} which does not exist',
+             node=self, assign_path=asspath, context=context))
 
-nodes.Tuple.assigned_stmts = mulass_assigned_stmts
-nodes.List.assigned_stmts = mulass_assigned_stmts
+    asspath.insert(0, index)
+    return self.parent.assigned_stmts(node=self, context=context, asspath=asspath)
+
+nodes.Tuple.assigned_stmts = sequence_assigned_stmts
+nodes.List.assigned_stmts = sequence_assigned_stmts
 
 
-def assend_assigned_stmts(self, context=None):
-    return self.parent.assigned_stmts(self, context=context)
+def assend_assigned_stmts(self, node=None, context=None, asspath=None):
+    return self.parent.assigned_stmts(node=self, context=context)
 nodes.AssignName.assigned_stmts = assend_assigned_stmts
 nodes.AssignAttr.assigned_stmts = assend_assigned_stmts
 
@@ -325,7 +337,7 @@ def _arguments_infer_argname(self, name, context):
         yield util.YES
 
 
-def arguments_assigned_stmts(self, node, context, asspath=None):
+def arguments_assigned_stmts(self, node=None, context=None, asspath=None):
     if context.callcontext:
         # reset call context/name
         callcontext = context.callcontext
@@ -339,7 +351,7 @@ nodes.Arguments.assigned_stmts = arguments_assigned_stmts
 
 
 @bases.raise_if_nothing_inferred
-def assign_assigned_stmts(self, node, context=None, asspath=None):
+def assign_assigned_stmts(self, node=None, context=None, asspath=None):
     if not asspath:
         yield self.value
         return
@@ -380,7 +392,7 @@ def _resolve_asspart(parts, asspath, context):
 
 
 @bases.raise_if_nothing_inferred
-def excepthandler_assigned_stmts(self, node, context=None, asspath=None):
+def excepthandler_assigned_stmts(self, node=None, context=None, asspath=None):
     for assigned in node_classes.unpack_infer(self.type):
         if isinstance(assigned, nodes.ClassDef):
             assigned = bases.Instance(assigned)
@@ -389,7 +401,7 @@ nodes.ExceptHandler.assigned_stmts = bases.raise_if_nothing_inferred(excepthandl
 
 
 @bases.raise_if_nothing_inferred
-def with_assigned_stmts(self, node, context=None, asspath=None):
+def with_assigned_stmts(self, node=None, context=None, asspath=None):
     if asspath is None:
         for _, vars in self.items:
             if vars is None:
