@@ -59,6 +59,12 @@ try:
 except ImportError:
     HAS_NUMPY = False
 
+try:
+    import pytest # pylint: disable=unused-import
+    HAS_PYTEST = True
+except ImportError:
+    HAS_PYTEST = False
+
 
 class HashlibTest(unittest.TestCase):
     def test_hashlib(self):
@@ -79,6 +85,7 @@ class HashlibTest(unittest.TestCase):
 
 
 class NamedTupleTest(unittest.TestCase):
+
     def test_namedtuple_base(self):
         klass = test_utils.extract_node("""
         from collections import namedtuple
@@ -104,7 +111,7 @@ class NamedTupleTest(unittest.TestCase):
         for base in klass.ancestors():
             if base.name == 'X':
                 break
-        self.assertSetEqual({"a", "b", "c"}, set(base.instance_attrs))
+        self.assertSetEqual({"a", "b", "c"}, set(base._instance_attrs))
 
     def test_namedtuple_inference_failure(self):
         klass = test_utils.extract_node("""
@@ -139,9 +146,19 @@ class NamedTupleTest(unittest.TestCase):
         namedtuple('a', 'a b c')(1, 2, 3) #@
         ''')
         inferred = next(result.infer())
-        for name, attr in inferred.instance_attrs.items():
+        for name, attr in inferred._instance_attrs.items():
             self.assertEqual(attr[0].attrname, name)
 
+    def test_namedtuple_uninferable_fields(self):
+        node = test_utils.extract_node('''
+        x = [A] * 2
+        from collections import namedtuple
+        l = namedtuple('a', x)
+        l(1)
+        ''')
+        inferred = next(node.infer())
+        self.assertIs(util.YES, inferred)
+
 
 class ModuleExtenderTest(unittest.TestCase):
     def testExtensionModules(self):
@@ -284,6 +301,15 @@ class MultiprocessingBrainTest(unittest.TestCase):
         else:
             self.assertIsInstance(cpu_count, astroid.BoundMethod)
 
+    def test_module_name(self):
+        module = test_utils.extract_node("""
+        import multiprocessing
+        multiprocessing.SyncManager()
+        """)
+        inferred_sync_mgr = next(module.infer())
+        module = inferred_sync_mgr.root()
+        self.assertEqual(module.name, 'multiprocessing.managers')
+
     def test_multiprocessing_manager(self):
         # Test that we have the proper attributes
         # for a multiprocessing.managers.SyncManager
@@ -337,6 +363,11 @@ class MultiprocessingBrainTest(unittest.TestCase):
         array = next(module['array'].infer())
         self.assertEqual(array.qname(), "array.array")
 
+        manager = next(module['manager'].infer())
+        # Verify that we have these attributes
+        self.assertTrue(manager.getattr('start'))
+        self.assertTrue(manager.getattr('shutdown'))
+
 
 @unittest.skipUnless(HAS_ENUM,
                      'The enum module was only added in Python 3.4. Support for '
@@ -414,6 +445,20 @@ class EnumBrainTest(unittest.TestCase):
         self.assertTrue(clazz.is_subtype_of(int_type),
                         'IntEnum based enums should be a subtype of int')
 
+    def test_enum_func_form_is_class_not_instance(self):
+        cls, instance = test_utils.extract_node('''
+        from enum import Enum
+        f = Enum('Audience', ['a', 'b', 'c'])
+        f #@
+        f(1) #@
+        ''')
+        inferred_cls = next(cls.infer())
+        self.assertIsInstance(inferred_cls, bases.Instance)
+        inferred_instance = next(instance.infer())
+        self.assertIsInstance(inferred_instance, bases.Instance)
+        self.assertIsInstance(next(inferred_instance.igetattr('name')), nodes.Const)
+        self.assertIsInstance(next(inferred_instance.igetattr('value')), nodes.Const)
+
 
 @unittest.skipUnless(HAS_DATEUTIL, "This test requires the dateutil library.")
 class DateutilBrainTest(unittest.TestCase):
@@ -438,5 +483,21 @@ class NumpyBrainTest(unittest.TestCase):
         self.assertIsInstance(inferred, nodes.FunctionDef)
 
 
+@unittest.skipUnless(HAS_PYTEST, "This test requires the pytest library.")
+class PytestBrainTest(unittest.TestCase):
+
+    def test_pytest(self):
+        ast_node = test_utils.extract_node('''
+        import pytest
+        pytest #@
+        ''')
+        module = next(ast_node.infer())
+        self.assertIn('deprecated_call', module)
+        self.assertIn('exit', module)
+        self.assertIn('fail', module)
+        self.assertIn('fixture', module)
+        self.assertIn('mark', module)
+
+
 if __name__ == '__main__':
     unittest.main()
