@@ -111,19 +111,41 @@ class ModuleNodeTest(ModuleLoader, unittest.TestCase):
         res = sorted(m.wildcard_import_names())
         self.assertEqual(res, ['Aaa', 'func', 'name', 'other'])
 
+    def test_public_names(self):
+        m = builder.parse('''
+        name = 'a'
+        _bla = 2
+        other = 'o'
+        class Aaa: pass
+        def func(): print('yo')
+        __all__ = 'Aaa', '_bla', 'name'
+        ''')
+        values = sorted(['Aaa', 'name', 'other', 'func'])
+        self.assertEqual(sorted(m._public_names()), values)
+        m = builder.parse('''
+        name = 'a'
+        _bla = 2
+        other = 'o'
+        class Aaa: pass
+
+        def func(): return 'yo'
+        ''')
+        res = sorted(m._public_names())
+        self.assertEqual(res, values)
+
         m = builder.parse('''
             from missing import tzop
             trop = "test"
             __all__ = (trop, "test1", tzop, 42)
         ''')
-        res = sorted(m.wildcard_import_names())
-        self.assertEqual(res, ["test", "test1"])
+        res = sorted(m._public_names())
+        self.assertEqual(res, ["trop", "tzop"])
 
         m = builder.parse('''
             test = tzop = 42
             __all__ = ('test', ) + ('tzop', )
         ''')
-        res = sorted(m.wildcard_import_names())
+        res = sorted(m._public_names())
         self.assertEqual(res, ['test', 'tzop'])
 
     def test_module_getattr(self):
@@ -285,7 +307,7 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
         '''
         tree = builder.parse(code)
         func = tree['nested_args']
-        self.assertEqual(sorted(func.locals), ['a', 'b', 'c', 'd'])
+        self.assertEqual(sorted(func._locals), ['a', 'b', 'c', 'd'])
         self.assertEqual(func.args.format_args(), 'a, (b, c, d)')
 
     def test_four_args(self):
@@ -451,14 +473,14 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
                 def stcmethod(cls):
                     pass
         """)
-        node = astroid.locals['Node'][0]
-        self.assertEqual(node.locals['clsmethod_subclass'][0].type,
+        node = astroid._locals['Node'][0]
+        self.assertEqual(node._locals['clsmethod_subclass'][0].type,
                          'classmethod')
-        self.assertEqual(node.locals['clsmethod'][0].type,
+        self.assertEqual(node._locals['clsmethod'][0].type,
                          'classmethod')
-        self.assertEqual(node.locals['staticmethod_subclass'][0].type,
+        self.assertEqual(node._locals['staticmethod_subclass'][0].type,
                          'staticmethod')
-        self.assertEqual(node.locals['stcmethod'][0].type,
+        self.assertEqual(node._locals['stcmethod'][0].type,
                          'staticmethod')
 
     def test_decorator_builtin_descriptors(self):
@@ -524,20 +546,20 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
                 def long_classmethod(cls): 
                     pass
         """)
-        node = astroid.locals['SomeClass'][0]
-        self.assertEqual(node.locals['static'][0].type,
+        node = astroid._locals['SomeClass'][0]
+        self.assertEqual(node._locals['static'][0].type,
                          'staticmethod')
-        self.assertEqual(node.locals['classmethod'][0].type,
+        self.assertEqual(node._locals['classmethod'][0].type,
                          'classmethod')
-        self.assertEqual(node.locals['not_so_static'][0].type,
+        self.assertEqual(node._locals['not_so_static'][0].type,
                          'method')
-        self.assertEqual(node.locals['not_so_classmethod'][0].type,
+        self.assertEqual(node._locals['not_so_classmethod'][0].type,
                          'method')
-        self.assertEqual(node.locals['classmethod_wrapped'][0].type,
+        self.assertEqual(node._locals['classmethod_wrapped'][0].type,
                          'classmethod')
-        self.assertEqual(node.locals['staticmethod_wrapped'][0].type,
+        self.assertEqual(node._locals['staticmethod_wrapped'][0].type,
                          'staticmethod')
-        self.assertEqual(node.locals['long_classmethod'][0].type,
+        self.assertEqual(node._locals['long_classmethod'][0].type,
                          'classmethod')
 
     def test_igetattr(self):
@@ -545,7 +567,7 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
         def test():
             pass
         ''')
-        func.instance_attrs['value'] = [nodes.Const(42)]
+        func._instance_attrs['value'] = [nodes.Const(42)]
         value = func.getattr('value')
         self.assertEqual(len(value), 1)
         self.assertIsInstance(value[0], nodes.Const)
@@ -554,6 +576,18 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
         self.assertIsInstance(inferred, nodes.Const)
         self.assertEqual(inferred.value, 42)
 
+    @test_utils.require_version(minver='3.0')
+    def test_return_annotation_is_not_the_last(self):
+        func = builder.parse('''
+        def test() -> bytes:
+            pass
+            pass
+            return
+        ''').body[0]
+        last_child = func.last_child()
+        self.assertIsInstance(last_child, nodes.Return)
+        self.assertEqual(func.tolineno, 5)
+
 
 class ClassNodeTest(ModuleLoader, unittest.TestCase):
 
@@ -584,6 +618,29 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
             self.assertEqual(len(cls.getattr('__dict__')), 1)
             self.assertEqual(len(cls.getattr('__mro__')), 1)
 
+    def test__mro__attribute(self):
+        node = test_utils.extract_node('''
+        class A(object): pass
+        class B(object): pass
+        class C(A, B): pass        
+        ''')
+        mro = node.getattr('__mro__')[0]
+        self.assertIsInstance(mro, nodes.Tuple)
+        self.assertEqual(mro.elts, node.mro())
+
+    def test__bases__attribute(self):
+        node = test_utils.extract_node('''
+        class A(object): pass
+        class B(object): pass
+        class C(A, B): pass
+        class D(C): pass
+        ''')
+        bases = node.getattr('__bases__')[0]
+        self.assertIsInstance(bases, nodes.Tuple)
+        self.assertEqual(len(bases.elts), 1)
+        self.assertIsInstance(bases.elts[0], nodes.ClassDef)
+        self.assertEqual(bases.elts[0].name, 'C')
+
     def test_cls_special_attributes_2(self):
         astroid = builder.parse('''
             class A: pass
@@ -753,7 +810,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         '''
         astroid = builder.parse(data, __name__)
         cls = astroid['WebAppObject']
-        self.assertEqual(sorted(cls.locals.keys()),
+        self.assertEqual(sorted(cls._locals.keys()),
                          ['appli', 'config', 'registered', 'schema'])
 
     def test_class_getattr(self):
@@ -952,9 +1009,9 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
                class D: pass
         """)
         a = astroid['A']
-        b = a.locals['B'][0]
+        b = a._locals['B'][0]
         c = astroid['C']
-        d = c.locals['D'][0]
+        d = c._locals['D'][0]
         self.assertEqual(a.metaclass().name, 'ABCMeta')
         self.assertFalse(b.newstyle)
         self.assertIsNone(b.metaclass())
@@ -1188,6 +1245,33 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         cls = module['Klass']
         self.assertEqual(cls.slots(), [])
 
+    def test_slots_taken_from_parents(self):
+        module = builder.parse('''
+        class FirstParent(object):
+            __slots__ = ('a', 'b', 'c')
+        class SecondParent(FirstParent):
+            __slots__ = ('d', 'e')
+        class Third(SecondParent):
+            __slots__ = ('d', )
+        ''')
+        cls = module['Third']
+        slots = cls.slots()
+        self.assertEqual(sorted(set(slot.value for slot in slots)),
+                         ['a', 'b', 'c', 'd', 'e'])
+
+    def test_all_ancestors_need_slots(self):
+        module = builder.parse('''
+        class A(object):
+            __slots__ = ('a', )
+        class B(A): pass
+        class C(B):
+            __slots__ = ('a', )
+        ''')
+        cls = module['C']
+        self.assertIsNone(cls.slots())
+        cls = module['B']
+        self.assertIsNone(cls.slots())
+
     def assertEqualMro(self, klass, expected_mro):
         self.assertEqual(
             [member.name for member in klass.mro()],
@@ -1345,87 +1429,6 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         type_cls = scoped_nodes.builtin_lookup("type")[1][0]
         self.assertEqual(cls.implicit_metaclass(), type_cls)
 
-    def test_implicit_metaclass_lookup(self):
-        cls = test_utils.extract_node('''
-        class A(object):
-            pass
-        ''')
-        instance = cls.instanciate_class()
-        func = cls.getattr('mro')
-        self.assertEqual(len(func), 1)
-        self.assertRaises(NotFoundError, instance.getattr, 'mro')
-
-    def test_metaclass_lookup_using_same_class(self):
-        # Check that we don't have recursive attribute access for metaclass
-        cls = test_utils.extract_node('''
-        class A(object): pass            
-        ''')
-        self.assertEqual(len(cls.getattr('mro')), 1)
-
-    def test_metaclass_lookup_inferrence_errors(self):
-        module = builder.parse('''
-        import six
-
-        class Metaclass(type):
-            foo = lala
-
-        @six.add_metaclass(Metaclass)
-        class B(object): pass 
-        ''')
-        cls = module['B']
-        self.assertEqual(util.YES, next(cls.igetattr('foo')))
-
-    def test_metaclass_lookup(self):
-        module = builder.parse('''
-        import six
-         
-        class Metaclass(type):
-            foo = 42
-            @classmethod
-            def class_method(cls):
-                pass
-            def normal_method(cls):
-                pass
-            @property
-            def meta_property(cls):
-                return 42
-            @staticmethod
-            def static():
-                pass
-
-        @six.add_metaclass(Metaclass)
-        class A(object):
-            pass
-        ''')
-        acls = module['A']
-        normal_attr = next(acls.igetattr('foo'))
-        self.assertIsInstance(normal_attr, nodes.Const)
-        self.assertEqual(normal_attr.value, 42)
-
-        class_method = next(acls.igetattr('class_method'))
-        self.assertIsInstance(class_method, BoundMethod)
-        self.assertEqual(class_method.bound, module['Metaclass'])
-
-        normal_method = next(acls.igetattr('normal_method'))
-        self.assertIsInstance(normal_method, BoundMethod)
-        self.assertEqual(normal_method.bound, module['A'])
-
-        # Attribute access for properties:
-        #   from the metaclass is a property object
-        #   from the class that uses the metaclass, the value
-        #   of the property
-        property_meta = next(module['Metaclass'].igetattr('meta_property'))
-        self.assertIsInstance(property_meta, UnboundMethod)
-        wrapping = scoped_nodes.get_wrapping_class(property_meta)
-        self.assertEqual(wrapping, module['Metaclass'])
-
-        property_class = next(acls.igetattr('meta_property'))
-        self.assertIsInstance(property_class, nodes.Const)
-        self.assertEqual(property_class.value, 42)
-
-        static = next(acls.igetattr('static'))
-        self.assertIsInstance(static, scoped_nodes.FunctionDef)
-
     @test_utils.require_version(maxver='3.0')
     def test_implicit_metaclass_is_none(self):
         cls = test_utils.extract_node("""
